import React, { useEffect, useState, useRef } from 'react';
import JSZip from 'jszip';

const Animation = ({ src = "M_Porsche_cross_arm.zip", h = '100%', w = '100%' }) => {
  const [images, setImages] = useState([]);
  const [loading, setLoading] = useState(true);
  const frameRate = 1000 / 24; // 24 FPS
  const canvasRef = useRef(null);
  const animationRef = useRef(null);
  const currentFrameRef = useRef(0);
  const imageElementsRef = useRef([]);
  const urlSource = `${import.meta.env.VITE_ASSETS_URL}/${src}`;

  useEffect(() => {
    const loadImages = async () => {
      const zip = new JSZip();
      try {
        const response = await fetch(urlSource);
        if (!response.ok) {
          throw new Error('Network response was not ok');
        }
        const data = await response.arrayBuffer();
        const zipContent = await zip.loadAsync(data);

        const imgPromises = [];
        zipContent.forEach((relativePath, file) => {
          if (file.name.endsWith('.webp')) {
            imgPromises.push(
              file.async('base64').then(base64 => {
                const img = new Image();
                img.src = `data:image/webp;base64,${base64}`;
                return new Promise((resolve, reject) => {
                  img.onload = () => resolve(img);
                  img.onerror = reject;
                });
              })
            );
          }
        });

        const imgElements = await Promise.all(imgPromises);
        if (imgElements.length === 0) {
          console.error('No images found in the ZIP file.');
        }

        imageElementsRef.current = imgElements;
        setImages(imgElements);
      } catch (error) {
        console.error('Error loading images:', error);
      } finally {
        setLoading(false);
      }
    };

    loadImages();
  }, [urlSource]);

  useEffect(() => {
    if (images.length > 0) {
      const canvas = canvasRef.current;
      const gl = canvas.getContext('webgl');
      if (!gl) {
        console.error("WebGL is not supported.");
        return;
      }

      const rect = canvas.getBoundingClientRect();
      const dpr = window.devicePixelRatio || 1;
      canvas.width = rect.width * dpr;
      canvas.height = rect.height * dpr;
      gl.viewport(0, 0, canvas.width, canvas.height);

      canvas.style.width = `${rect.width}px`;
      canvas.style.height = `${rect.height}px`;

      gl.clearColor(0.0, 0.0, 0.0, 0.0);
      gl.clear(gl.COLOR_BUFFER_BIT);

      gl.enable(gl.BLEND);
      gl.blendFunc(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA);

      const vertexShaderSource = `
        attribute vec2 a_position;
        attribute vec2 a_texCoord;
        varying vec2 v_texCoord;
        void main() {
          gl_Position = vec4(a_position, 0.0, 1.0);
          v_texCoord = a_texCoord;
        }
      `;
      const fragmentShaderSource = `
        precision mediump float;
        uniform sampler2D u_texture;
        varying vec2 v_texCoord;
        void main() {
          gl_FragColor = texture2D(u_texture, v_texCoord);
        }
      `;

      const createShader = (source, type) => {
        const shader = gl.createShader(type);
        gl.shaderSource(shader, source);
        gl.compileShader(shader);
        if (!gl.getShaderParameter(shader, gl.COMPILE_STATUS)) {
          console.error('Shader compile error:', gl.getShaderInfoLog(shader));
        }
        return shader;
      };

      const vertexShader = createShader(vertexShaderSource, gl.VERTEX_SHADER);
      const fragmentShader = createShader(fragmentShaderSource, gl.FRAGMENT_SHADER);

      const shaderProgram = gl.createProgram();
      gl.attachShader(shaderProgram, vertexShader);
      gl.attachShader(shaderProgram, fragmentShader);
      gl.linkProgram(shaderProgram);
      if (!gl.getProgramParameter(shaderProgram, gl.LINK_STATUS)) {
        console.error('Program link error:', gl.getProgramInfoLog(shaderProgram));
      }
      gl.useProgram(shaderProgram);

      const positionBuffer = gl.createBuffer();
      gl.bindBuffer(gl.ARRAY_BUFFER, positionBuffer);
      const vertices = new Float32Array([
        -1.0, -1.0,
         1.0, -1.0,
        -1.0,  1.0,
         1.0,  1.0,
      ]);
      gl.bufferData(gl.ARRAY_BUFFER, vertices, gl.STATIC_DRAW);

      const positionLocation = gl.getAttribLocation(shaderProgram, "a_position");
      gl.vertexAttribPointer(positionLocation, 2, gl.FLOAT, false, 0, 0);
      gl.enableVertexAttribArray(positionLocation);

      const texCoordBuffer = gl.createBuffer();
      gl.bindBuffer(gl.ARRAY_BUFFER, texCoordBuffer);
      const texCoords = new Float32Array([
        0.0, 1.0,
        1.0, 1.0,
        0.0, 0.0,
        1.0, 0.0,
      ]);
      gl.bufferData(gl.ARRAY_BUFFER, texCoords, gl.STATIC_DRAW);

      const texCoordLocation = gl.getAttribLocation(shaderProgram, "a_texCoord");
      gl.vertexAttribPointer(texCoordLocation, 2, gl.FLOAT, false, 0, 0);
      gl.enableVertexAttribArray(texCoordLocation);

      const texture = gl.createTexture();
      gl.bindTexture(gl.TEXTURE_2D, texture);
      gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
      gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
      gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);
      gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
      // Optionally use NEAREST for sharper image:
      // gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
      // gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);

      const uTextureLocation = gl.getUniformLocation(shaderProgram, "u_texture");

      const loadTexture = (image) => {
        gl.bindTexture(gl.TEXTURE_2D, texture);
        gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, image);
        gl.generateMipmap(gl.TEXTURE_2D);
      };

      loadTexture(imageElementsRef.current[0]);

      let lastFrameTime = 0;

      const animate = (timestamp) => {
        if (lastFrameTime === 0) lastFrameTime = timestamp;

        const elapsed = timestamp - lastFrameTime;
        if (elapsed > frameRate) {
          currentFrameRef.current = (currentFrameRef.current + 1) % images.length;
          loadTexture(imageElementsRef.current[currentFrameRef.current]);
          lastFrameTime = timestamp;
        }

        gl.clear(gl.COLOR_BUFFER_BIT);
        gl.drawArrays(gl.TRIANGLE_STRIP, 0, 4);

        animationRef.current = requestAnimationFrame(animate);
      };

      animationRef.current = requestAnimationFrame(animate);

      return () => {
        if (animationRef.current) {
          cancelAnimationFrame(animationRef.current);
        }
      };
    }
  }, [images]);

  if (loading) {
    return <div>Loading...</div>;
  } else {
    return (
      <canvas
        ref={canvasRef}
        style={{ height: h, width: w }}
      />
    );
  }
};

export default Animation;
